/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
package jdct;

import jdct.*;
import dbStatic.* ;
import java.lang.*;
import java.awt.*;

class JdctRecordInstance extends Panel
{
	// remove Frame from this class later
	private JdctFrame frame;

	private int windowId;
	private String windowName;
	private String typeName = "xxxx";
	private String recordName = "yyyy";
	private JdctInstanceGui instanceGui;

	private JdctLinkFieldInstance linkInstance;
	private JdctMenuFormFieldInstance menuFormInstance;


	JdctRecordInstance( JdctFrame frame, int windowIdIn)
	    {
		this.frame = frame;
		typeName = new String("                   ");
		recordName = new String("No Instance loaded          ");
		instanceGui = new JdctInstanceGui(this);
		instanceGui.addFocusListener(frame);
		windowId = windowIdIn;
		instanceGui.setTitle("Window No. " + windowId);
	}

	synchronized void loadInstance(String recordName)
	{
		if (recordName == null) {
			recordName = new String("No instance loaded          ");
			frame.setRecordInstance("  ",recordName);
			instanceGui.setInstanceLabel(recordName);
			instanceGui.setNFields(0);
			instanceGui.pack();
			instanceGui.setVisible(true);
			instanceGui.toFront();
			return;
		}

		DbBase dbBase = frame.getDbBase();
		DbEntry dbEntry = new DbEntry(dbBase);
		this.recordName = recordName;
		if(!dbEntry.findRecord(recordName)) {
			instanceGui.messageDialog("ERROR",
			    "Record " + recordName + " Not Found?",new Point(100,100));
			dbEntry.free();
			return;
		}

		this.typeName = dbEntry.getRecordTypeName();
		frame.setRecordInstance(typeName,recordName);
		instanceGui.setInstanceLabel(typeName + ":   " + recordName);
		int nfields = dbEntry.getNFields(true);
		if(nfields<=0) return;
		//System.out.println("for record: " + recordName + "  no.of fields = " + nfields);
		instanceGui.setNFields(nfields);

		// Check to see if this record has DTYP field
		    boolean  dtypExists = false;
                boolean relFieldExists  = false;
		String  relFieldName = "b1";
		int     relFieldNum  = 9999 ;
		String  relFieldValue= "b2" ;
		String  relFieldDesc = "b3" ;
		int     relFieldType = 999 ;
		String  fieldName    = "b4" ;
		String  desc         = "b5" ;
		int     type         = 998 ;


		dbEntry.firstField(true);

		for(int i=0; i<nfields; i++, dbEntry.nextField(true)) {

			type = dbEntry.getFieldType();
			// Search for DTYP record ...
			    if (type == DbEntry.DCT_MENUFORM) {
                                try{
				relFieldName = dbEntry.getRelatedField();
                                dtypExists = true;
                                relFieldExists = true;
                                }
                                catch (Exception e) {
                                relFieldName = "none";
                                dtypExists = false;
                                relFieldExists = false;
                                }
  //System.out.println(" for dtype field, the relFieldName=" + relFieldName + "*****");
			
			}
			fieldName = dbEntry.getFieldName();
			if (fieldName.equals(relFieldName)) {
				relFieldNum  = i ;
				relFieldValue = dbEntry.getString();
				relFieldDesc = dbEntry.getPrompt();
				relFieldType = dbEntry.getFieldType();
				break;
			}

		}

		int skipRelField = relFieldNum + 1;

		// get back to 1st field of record
		    dbEntry.firstField(true);

		for(int i=0; i<nfields; i++, dbEntry.nextField(true)) {

			if (i == skipRelField)  dbEntry.nextField(true) ;

// the data value associated with the current field is:
			String  origValue = dbEntry.getString();

			fieldName = dbEntry.getFieldName();
			instanceGui.setFieldName(i,fieldName);

			type = dbEntry.getFieldType();
			instanceGui.setFieldType(i,DbEntry.typeNames[type]);

			desc = dbEntry.getPrompt();
			instanceGui.setFieldDesc(i,desc);


			if(type==DbEntry.DCT_MENU || type==DbEntry.DCT_MENUFORM) {
				int nchoices = dbEntry.getNMenuChoices();
				if(nchoices<=0) return;
				String[] choices = new String [nchoices];
				for(int j=0; j<nchoices; j++) {
					choices[j] = dbEntry.getMenuStringFromIndex(j);
				}
				if(type==DbEntry.DCT_MENUFORM) {
					menuFormInstance = instanceGui.addMenuFormField(frame,
					    recordName,fieldName,choices,dbEntry.getString());
				}
				else instanceGui.addMenuField(frame, recordName,
				    fieldName,choices,dbEntry.getString());
			} else if(type==DbEntry.DCT_INLINK  ||
			    type==DbEntry.DCT_OUTLINK ||
			    type==DbEntry.DCT_FWDLINK  ) {
				instanceGui.addLinkField(frame,
				    recordName,fieldName,dbEntry.getString(),
				    dbEntry.getLinkType());
			} else {
				instanceGui.addTextField(frame,recordName,
				    fieldName,dbEntry.getString());
			}

			if (type == DbEntry.DCT_MENUFORM  & relFieldExists) {

				//String  origValue = dbEntry.getString();
				int ip = i + 1 ;


				fieldName = relFieldName;
				instanceGui.setFieldName(ip,fieldName);

				type = relFieldType;
				instanceGui.setFieldType(ip,DbEntry.typeNames[type]);
				desc = relFieldDesc;
				instanceGui.setFieldDesc(ip,desc);

				linkInstance = instanceGui.addLinkField(frame,
				    recordName,fieldName,relFieldValue,
				    dbEntry.getLinkType());
				menuFormInstance.setRelatedFieldInstance(linkInstance);
				i = ip ;
			}
		}
		instanceGui.pack();
		dbEntry.free();
	}

	String getTypeName()
	    {
		return typeName;
	}
	String getRecordName()
	    {
		return recordName;
	}
	String getWindowName()
	    {
		return instanceGui.getTitle();
	}
	int getWindowId()
	{
		return windowId;
	}
	JdctInstanceGui getInstanceGUI()
	{
		return instanceGui;
	}
	void removeRecordInstance()
	{
		frame.removeRecordInstance(this);
		instanceGui = null;
	}
	void activate()
	{
		instanceGui.show();
		instanceGui.toFront();
	}
}
